unit Dirs;
{$I-}
interface
uses
   DOS,
   Objects;

const
   rdOk =          0;
   rdError =       1;

type
   TRecurseDirs =
   object(TObject)
      Status       :Word;
      Dir          :DirStr;
      Info         :SearchRec;

      procedure Run(const StartDir     :String);

      procedure ProcessDir; virtual;
      procedure PostProcessDir; virtual;

   private
      InfoPtr      :^SearchRec;
   end;

   TProcessFiles =
   object(TRecurseDirs)
      FileMask     :String[12];

      constructor Init;

      procedure ProcessDir; virtual;
      procedure ProcessFile; virtual;
   end;

function  RenameFile(const SrcN, DstN  :String)  :Boolean;
function  CopyDir(const SrcD, DstD     :String)  :Boolean;
function  EraseDir(const Dir           :String)  :Boolean;

implementation

{ TRecurseDirs }

procedure TRecurseDirs.Run;

procedure ProcessSubdirs;
var
   Subdirs         :PCollection;
   DirIndex        :Integer;
begin
   ProcessDir;
   if Status<>rdOk then Exit;

   New(Subdirs, Init(0, 4));
   FindFirst(Dir+'*.*', Directory, Info);
   while DosError=0 do
   begin
      if (Info.Attr and Directory<>0) and (Info.Name[1]<>'.') then
      begin
         New(InfoPtr); InfoPtr^:=Info;
         Subdirs^.Insert(InfoPtr);
      end;
      FindNext(Info);
   end;

   if DosError<>18 then Status:=rdError else
   for DirIndex:=0 to Subdirs^.Count-1 do
   begin
      InfoPtr:=Subdirs^.At(DirIndex);
      Info:=InfoPtr^;
      Dispose(InfoPtr);

      Dir:=Dir+Info.Name+'\';
      asm add sp,100h end;
      ProcessSubdirs;
      asm sub sp,100h end;

      if Dir<>'' then
      repeat
         Dec(Dir[0]);
      until (Dir[Length(Dir)]='\') or (Dir='');
   end;

   Subdirs^.DeleteAll;
   Dispose(Subdirs, Done);

   PostProcessDir;

   DosError:=0;
end;

begin
   Dir:=StartDir;
   Status:=rdOk;
   ProcessSubdirs;
end;

procedure TRecurseDirs.ProcessDir;
begin
   Abstract;
end;

procedure TRecurseDirs.PostProcessDir;
begin
end;

{ TProcessFiles }

constructor TProcessFiles.Init;
begin
   Inherited Init;

   FileMask:='*.*';
end;

procedure TProcessFiles.ProcessDir;
var
   Files           :PCollection;
   FileIndex       :Integer;
begin
   New(Files, Init(0, 16));
   FindFirst(Dir+FileMask, AnyFile, Info);
   while DosError=0 do
   begin
      if (Info.Attr and (Directory or VolumeId))=0 then
      begin
         New(InfoPtr); InfoPtr^:=Info;
         Files^.Insert(InfoPtr);
      end;
      FindNext(Info);
   end;

   if DosError<>18 then Status:=rdError else
   for FileIndex:=0 to Files^.Count-1 do
   begin
      InfoPtr:=Files^.At(FileIndex);
      Info:=InfoPtr^;
      Dispose(InfoPtr);

      if Status=stOk then ProcessFile;
   end;

   Files^.DeleteAll;
   Dispose(Files, Done);
end;

procedure TProcessFiles.ProcessFile;
begin
   Abstract;
end;


function  RenameFile;
var
   F               :File;
begin
   Assign(F, SrcN); Rename(F, DstN);
   RenameFile:=(IOResult<>0);
end;

type
   TCopyDirs =
   object(TProcessFiles)
      DstStart     :PString;
      DstFull      :DirStr;
      SrcLen       :Integer;

      procedure ProcessDir; virtual;
      procedure ProcessFile; virtual;
   end;

procedure TCopyDirs.ProcessDir;
begin
   DstFull:=DstStart^+Copy(Dir, SrcLen+1, 255);
   Dec(DstFull[0]); MkDir(DstFull); Inc(DstFull[0]);
   if InOutRes=5 then InOutRes:=0;
   if IOResult<>0 then Status:=rdError else Inherited ProcessDir;
end;

procedure TCopyDirs.ProcessFile;
var
   SrcF, DstF      :TBufStream;
   BufSize         :Word;
   FreeSize        :LongInt;
begin
   FreeSize:=MaxAvail shr 2;
   if FreeSize>32768 then FreeSize:=32768;
   BufSize:=1024;
   while BufSize<FreeSize do asm shl BufSize,1 end;

   SrcF.Init(Dir+Info.Name, stOpenRead, BufSize);
   DstF.Init(DstFull+Info.Name, stCreate, BufSize);
   DstF.CopyFrom(SrcF, SrcF.GetSize);
   DstF.Done;
   SrcF.Done;
   if (SrcF.Status<>stOk) or (DstF.Status<>stOk) then Status:=rdError;
end;

function  CopyDir;
var
   R               :TCopyDirs;
begin
   R.Init;
   R.DstStart:=@DstD;
   R.SrcLen:=Length(SrcD);
   R.Run(SrcD+'\');
   CopyDir:=(R.Status<>rdOk);
end;

type
   TEraseDirs =
   object(TProcessFiles)
      procedure PostProcessDir; virtual;
      procedure ProcessFile; virtual;
   end;

procedure TEraseDirs.PostProcessDir;
begin
   Dec(Dir[0]); RmDir(Dir); Inc(Dir[0]);
   if IOResult<>0 then Status:=rdError;
end;

procedure TEraseDirs.ProcessFile;
var
   F               :File;
begin
   Assign(F, Dir+Info.Name); Erase(F);
   if IOResult<>0 then Status:=rdError;
end;

function  EraseDir;
var
   R               :TEraseDirs;
begin
   R.Init;
   R.Run(Dir+'\');
   EraseDir:=(R.Status<>rdOk);
end;

end.

